/*

 * WordNet-Java

 *

 * Copyright 1998 by Oliver Steele.  You can use this software freely so long as you preserve

 * the copyright notice and this restriction, and label your changes.

 */

package edu.brandeis.cs.steele.wn;
 import  edu.brandeis.cs.steele.wn.util.*;
import java.util.*;



/** The methods in this Interface must be propertly  implemented to access the Wordnet
 * database.  This method was chosen to allow interaction between a local file or remote
 * file system for database interaction.
 * @see FileBackedDictionary
 */

public interface DictionaryDatabase {

	/** Look up a word in the database.  The search is case-independent,
	 * and phrases are separated by spaces ("look up", not "look_up").
	 * @param pos The part-of-speech.
	 * @param lemma The orthographic representation of the word.
	 * @return An IndexWord representing the word, or <code>null</code> if no such entry exists.
	 */

    public IndexWord lookupIndexWord(POS pos, String lemma);



    /** Searches the base word <i>(lemma)</i> of the given string in the database.
     *
     * @param pos The part-of-speech.
     * @param derivation The inflected form of the word.
     * @return The uninflected word, or null if no exception entry exists.
    */

    public String lookupBaseForm(POS pos, String derivation);



    /** Return an enumeration of all the IndexWords whose lemmas contain <var>substring</var>

     * as a substring.

	 * @param pos The part-of-speech.

	 * @return An enumeration of <code>IndexWord</code>s.

	 */

	public Enumeration searchIndexWords(POS pos, String substring);



	/** Return an enumeration over all the Synsets in the database.

	 * @param pos The part-of-speech.

	 * @return An enumeration of <code>Synset</code>s.

	 */

	public Enumeration synsets(POS pos);

}
