package edu.gwu.jworddrivers;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.*;



/**
 *
 * @author  Kunal Johar
 * Please not that this is an abstract class and all its methods must be
 * overridden in order to provide functionality.  The JWordDriver combined with
 * the JWordNode provide a mechanism to bridge various word-relational databases
 * with the JWord viewing interface
 *
 */
public abstract class JWordDriver
{
    public static final int POS_NOUN = 0;
    public static final int POS_VERB = 1;
    public static final int POS_ADJECTIVE = 2;
    public static final int POS_ADVERB = 3;


    /** This method should return the title of the dictionary
     *
     */
    public abstract String getTitle();

    /** This method should be implemented to load all dictionary resources
     * this method will be called by JWord before running any other
     * methods in the class
     */
    public abstract void initDictionary() ;

    /** Given the starting position on a JTree and a search word
     * this method will construct DefatultMutableTreeNodes containing JWordNodes
     * based on the dictionary's word relationships matched by the searchWord
     */
    public abstract void  returnSearch(DefaultMutableTreeNode root, String searchWord);

    /** This method is identical to that of returnSearch except that it generates
     * the tree starting at the Part of Speech level rather than the dictionary level
     */
    public abstract void returnPOS(DefaultMutableTreeNode posRoot, int POS, String searchWord);

}
