/*
 * rogetParser.java
 *
 * Created on September 22, 2004, 9:22 PM
 */
package edu.gwu.jworddrivers;

import java.util.regex.*;
import java.util.*;
import java.io.*;
import java.nio.*;
import java.nio.charset.*;
import java.nio.channels.*;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.*;


/**
 *
 * @author  Kunal Johar
 * rogetParser uses a roget15a.txt Input File and Attempts to read it in
 */
public class RogetDriver extends JWordDriver
{
    private static String fileName;
    static final int STATE_NOUN = 0;
    static final int STATE_VERB = 1;
    static final int STATE_ADJ = 2;
    static final int STATE_ADV = 3;

    public static final int MODE_ALLWORDS = 0;
    public static final int MODE_HEADWORDS = 1;
    public static final int MODE_POS_NOUN = 0;
    public static final int MODE_POS_VERB = 1;
    public static final int MODE_POS_ADJECTIVE = 2;
    public static final int MODE_POS_ADVERB = 3;

    public String getTitle()
    {
      return "Roget's Thesaurus";
    }

    /** Load the dictionary file into memory */
    public void initDictionary()
    {
        //#[0-9]*\.\s*HEADWORD\.[^#]*
        //#[^#]*WORD\.[^#]*

        //Save file path
        //this.fileName = fileName;

        //Test if File Exists

        java.util.Properties p = new Properties();
        try
        {
            java.io.FileInputStream f = new java.io.FileInputStream("wn.props");
            p.load(f);
        }
        catch (Exception e)
        {
            System.out.println("Dictionary not found");
            System.exit(0);
        }
        fileName = p.getProperty("roget.path");
        fileName = fileName.replace('\\', '/');

        File testFile;
        System.out.println("Attempting to Load Dictionary...");
        testFile = new File(fileName);
        if (!testFile.exists())
        {
           System.out.println("Cannot Load Dictionary");
           System.exit(0);
        }
    }

    /** Use the MODE_ constants to set these flags
     * determines how thorough returnSearch should be
     */
    public  void changeSearchMode(int wordLimiter, int posLimiter)
    {

    }

    /** This method is identical to that of returnSearch except that it generates
     * the tree starting at the Part of Speech level rather than the dictionary level
     */
    public  void returnPOS(DefaultMutableTreeNode posRoot, int POS, String searchWord)
    {
      getWordInfoPOS(posRoot, searchWord, POS);
    }

    /** Given a DefaultMutableTreeNode to start at and a word to search for
     * this will build the tree based on the thesauras's word relationship
     * using the
     */
    public  void  returnSearch(DefaultMutableTreeNode root, String searchWord)
    {
       getWordInfo(root, searchWord);
    }

    private  String parseBlock(String rogetEntry, ArrayList[] POS, String searchWord)
    {
        /*DefaultMutableTreeNode root = new DefaultMutableTreeNode("Root");
        //create storage on the tree
        DefaultMutableTreeNode posNode;
        DefaultMutableTreeNode curLeaf;
        */

        boolean haveHeadWord = false;
        String curToken;
        String headWord;
        String curLine = "";

        String[] headSplit = rogetEntry.split("--", 2);
        headWord = headSplit[0];
        if (headSplit.length != 2)
        {
           for (int i = 0; i < 4; i++)
             POS[i] = null;
           return "";
        }

        //System.out.println(headSplit[1]);
        StringTokenizer st = new java.util.StringTokenizer(headSplit[1], "\n\t\f\r ");


        int curState = -1;

        ArrayList[] lists = new ArrayList[4];
        for (int i = 0; i < 4; i++)
        {
            lists[i] = new ArrayList();
        }

        while (st.hasMoreElements())
        {
            curToken = (String)st.nextElement();

            if (curToken.endsWith(";"))
            {
               curToken = curToken.replaceAll(";", "");
               curLine += curToken + " ";
               if (!curLine.equals("") && curState >= 0)
                  lists[curState].add(curLine);
               curLine = "";
            }
            else if (curToken.equals("N."))
            {
                if (!curLine.equals(""))
                    lists[curState].add(curLine);

                curState = STATE_NOUN;
                curLine = "";
            }
            else if (curToken.equals("V."))
            {
                if (!curLine.equals(""))
                    lists[curState].add(curLine);
                curState = STATE_VERB;
                curLine = "";

            }
            else if (curToken.equals("Adj."))
            {
                if (!curLine.equals(""))
                    lists[curState].add(curLine);
                curState = STATE_ADJ;
                curLine = "";
            }
            else if (curToken.equals("Adv."))
            {
                if (!curLine.equals(""))
                    lists[curState].add(curLine);
                curState = STATE_ADV;
                curLine = "";
            }
            else if (curToken.equals("<--") || curToken.equals("%"))
                //end of input
            {
                if (!curLine.equals("") && curState >= 0)
                    lists[curState].add(curLine);
                break;
            }
            else
            {
                if (curState >= 0)
                {
                    curLine += curToken + " ";
                }
            }
        }

        if (!curLine.equals("") && curState >= 0)
           lists[curState].add(curLine);


        for (int i = 0; i < 4; i++)
        {
            boolean addList = false;
            //System.out.println("POS:" + i);
            for (int j = 0; j < lists[i].size(); j++)
            {
                if( ((String)(lists[i].get(j))).toLowerCase().indexOf(searchWord.toLowerCase()) > -1)
                 {
                    addList = true;
                    break;
                 }
            }
              if (!addList)
                lists[i] = null;

                POS[i] = lists[i];
                //System.out.println("-----" + (String)lists[i].get(j));
        }

        return headWord;

    }

    /**
      * Perform a search only looking at headwords as key values
    **/
    public  void getHeadWordInfo(DefaultMutableTreeNode root, String word)
    {
        //#[0-9]*\.\s*HEADWORD\.[^#]*
        int options = 0;
        options |= Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE | Pattern.MULTILINE;

        Pattern lookup = Pattern.compile("#[0-9]*\\.\\s*" + word + "\\.[^#]*", options);

        CharBuffer cb = createFileCharSequence();

        Matcher m = lookup.matcher(cb);


        String retStr = null;
        while (m.find())
        {
            retStr = m.group();
           //System.out.println(m.group());
        }

        System.out.println("\nFound " + m.groupCount() + " Headwords");
        //return retStr;

    }

    /** Perform a search using any text found in the dictionary*/
    private void getWordInfoPOS(DefaultMutableTreeNode root, String word, int pos)
    {
        //DefaultMutableTreeNode root = new DefaultMutableTreeNode("Root");
        //DefaultTreeModel tm = new DefaultTreeModel(root);

        DefaultMutableTreeNode posNode = root;
        DefaultMutableTreeNode curSenseNode;

        //#[0-9]*\.\s*HEADWORD\.[^#]*
        ////#[^#]*WORD\.[^#]*

        int options = 0;
        options |= Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE | Pattern.MULTILINE;

        Pattern lookup = Pattern.compile("#[^#]*[\\s\\.;,]" + word +"[^#]*", options);

        CharBuffer cb = createFileCharSequence();

        Matcher m = lookup.matcher(cb);

        ArrayList headWords = new ArrayList();
        ArrayList matches = new ArrayList();



        String retStr = null;
        int count = 0;
        while (m.find())
        {
          //System.out.println(m.group());
           ArrayList[] POS = new ArrayList[4];
           count++;
           retStr = m.group();
           headWords.add(parseBlock(retStr, POS, word));
           matches.add(POS);
        }

        String[] POS_Type = {"Noun", "Verb", "Adjective", "Adverb"};


        int i  = pos;

            //System.out.println("POS: " + i);

            for (int j = 0; j < headWords.size(); j++)
            {
                ArrayList[] curMatch = (ArrayList[])matches.get(j);
                ArrayList curWordsDisp = curMatch[i];


                if (curWordsDisp != null)
                {
                    String lastStr = "";

                    curSenseNode = new DefaultMutableTreeNode(new RogetNode((String)headWords.get(j), word));
                    //System.out.println("--" + headWords.get(j));
                    for (int k = 0; k < curWordsDisp.size(); k++)
                    {
                        String curOut = (String)curWordsDisp.get(k);
                        if (!lastStr.equals(curOut))
                        {
                            if (curOut.indexOf(word) > -1)
                            {
                                String ts = "<html>";
                                curOut = curOut.replaceAll(word, "<b><i>" + word + "</b></i>");
                                curOut = ts + curOut + "</html>";

                                curSenseNode.add(new DefaultMutableTreeNode(curOut));
                            }
                            else
                            {
                             // curSenseNode.add(new DefaultMutableTreeNode(curOut));
                            }
                           //System.out.println("----" + curWordsDisp.get(k));
                           lastStr = (String)curWordsDisp.get(k);
                        }
                    }
                    posNode.add(curSenseNode);
                }
            }


    }


    /** Perform a search using any text found in the dictionary*/
    private void getWordInfo(DefaultMutableTreeNode root, String word)
    {
        //DefaultMutableTreeNode root = new DefaultMutableTreeNode("Root");
        //DefaultTreeModel tm = new DefaultTreeModel(root);

        DefaultMutableTreeNode posNode = null;
        DefaultMutableTreeNode curSenseNode;

        //#[0-9]*\.\s*HEADWORD\.[^#]*
        ////#[^#]*WORD\.[^#]*

        int options = 0;
        options |= Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE | Pattern.MULTILINE;

        Pattern lookup = Pattern.compile("#[^#]*[\\s\\.;,]" + word +"[^#]*", options);

        CharBuffer cb = createFileCharSequence();

        Matcher m = lookup.matcher(cb);

        ArrayList headWords = new ArrayList();
        ArrayList matches = new ArrayList();



        String retStr = null;
        int count = 0;
        while (m.find())
        {
          //System.out.println(m.group());
           ArrayList[] POS = new ArrayList[4];
           count++;
           retStr = m.group();
           headWords.add(parseBlock(retStr, POS, word));
           matches.add(POS);
        }

        String[] POS_Type = {"Noun", "Verb", "Adjective", "Adverb"};

        for (int i = 0; i < 4; i++)
        {
            posNode = new DefaultMutableTreeNode(POS_Type[i] + " Matches");
            //System.out.println("POS: " + i);

            for (int j = 0; j < headWords.size(); j++)
            {
                ArrayList[] curMatch = (ArrayList[])matches.get(j);
                ArrayList curWordsDisp = curMatch[i];


                if (curWordsDisp != null)
                {
                    String lastStr = "";

                    curSenseNode = new DefaultMutableTreeNode(new RogetNode((String)headWords.get(j), word));
                    //System.out.println("--" + headWords.get(j));
                    for (int k = 0; k < curWordsDisp.size(); k++)
                    {
                        String curOut = (String)curWordsDisp.get(k);
                        if (!lastStr.equals(curOut))
                        {
                            if (curOut.indexOf(word) > -1)
                            {
                                String ts = "<html>";
                                curOut = curOut.replaceAll(word, "<b><i>" + word + "</b></i>");
                                curOut = ts + curOut + "</html>";

                                curSenseNode.add(new DefaultMutableTreeNode(curOut));
                            }
                            else
                            {
                              //   curSenseNode.add(new DefaultMutableTreeNode(curOut));
                            }
                           //System.out.println("----" + curWordsDisp.get(k));
                           lastStr = (String)curWordsDisp.get(k);
                        }
                    }
                    posNode.add(curSenseNode);
                }
            }

            if(posNode != null)
              if (posNode.children().hasMoreElements())
               root.add(posNode);
        }
    }

    public  void printHeadWords()
    {
        //#[0-9]*\.\s*[^\.]*\.
        int options = 0;
        options |= Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE | Pattern.MULTILINE;

        Pattern headWords = Pattern.compile("#[0-9]*\\.\\s*[^\\.]*\\.", options);
        // Get a Channel for the source file
        CharBuffer cb = createFileCharSequence();

        // Run some matches
        Matcher m = headWords.matcher(cb);
       // while (m.find())
            //System.out.println("HWord: "+m.group());

    }

    private  CharBuffer createFileCharSequence()
    {
        //Creates a character sequence from the dictionary file
        try
        {
            File f;
            FileInputStream fis;
            FileChannel fc = null;

            f = new File(fileName);
            fis = new FileInputStream(f);
            fc = fis.getChannel();

            // Get a CharBuffer from the source file
            ByteBuffer bb = fc.map(FileChannel.MapMode.READ_ONLY,  0, (int)fc.size());
            Charset cs = Charset.forName("8859_1");
            CharsetDecoder cd = cs.newDecoder();
            return cd.decode(bb);
        }
        catch (Exception e)
        {
            return null;
        }

    }

}
