package edu.gwu.jworddrivers;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.*;
import java.util.*;
import java.io.*;

public class RootBookDriver extends JWordDriver
{
  private ArrayList dictionary;
  private Hashtable search;
  private int wordCount = 0;

   /** This method should return the title of the dictionary
     *
     */
    public String getTitle()
    {
      return "Roots Book Driver";
    }

    /** This method should be implemented to load all dictionary resources
     * this method will be called by JWord before running any other
     * methods in the class
     */
    public void initDictionary()
    {
       java.util.Properties p = new Properties();
       try
       {
           java.io.FileInputStream f = new java.io.FileInputStream("wn.props");
           p.load(f);
       }
       catch (Exception e)
       {
           System.out.println("Dictionary not found");
           System.exit(0);
       }
       String fileName = p.getProperty("rootbook.path");
       fileName = fileName.replace('\\', '/');

       File testFile;
       System.out.println("Attempting to Load Dictionary...");
       testFile = new File(fileName);
       if (!testFile.exists())
       {
          System.out.println("Cannot Load Dictionary");
          System.exit(0);
       }

       dictionary = new ArrayList();
       search = new Hashtable();

       parseFile(testFile);
       System.out.println("Dictionary Size: " + dictionary.size());
       System.out.println("Words Indexed: " + wordCount);

       //System.out.println("Searching for entropy");

       //ArrayList results = (ArrayList)search.get("entropy");
       //if (results != null)
       //System.out.println("Result: " +  results.size() + "\n" + results.get(0));

    }

    /* Reads in Dictionary from File, stores into arraylist dictionary*/
    private void parseFile(File dictionaryFile)
    {
      rootItem curEntry = null;

      try
      {
        BufferedReader in = new BufferedReader(new FileReader(dictionaryFile));
        String str;
        while ((str = in.readLine()) != null)
        {
            //Process current line

            //trim the current line
            str = str.trim();

            //ignore single alphabet lines
            if (str.length() <= 1)
              continue;

            //new root
            if (str.indexOf(":") > -1 && str.indexOf("") == -1)
            {
              String[] parts = str.split(":", 2);
              if (parts.length != 2)
                continue;

              //create new route
              if (curEntry != null)
                dictionary.add(curEntry);

              curEntry = new rootItem(parts[0], parts[1]);
            }
            else if (str.indexOf(":") > -1)
            {
              if (curEntry == null)
                continue;

              String[] parts = str.split(":", 2);
              if (parts.length != 2)
                continue;

              parts = parts[1].split(",");
              for (int i = 0; i < parts.length; i++)
              {
                parts[i] = parts[i].trim().replaceAll("", "").toLowerCase();
                curEntry.addWord(parts[i]);
                wordCount++;
                ArrayList check;

                check = (ArrayList) search.get(parts[i]);

                if (check == null) {
                  check = new ArrayList();
                }

                check.add(curEntry);

                search.put(parts[i], check);

              }
            }
            else
            {
              if (curEntry == null)
                continue;

              String parts[] = str.split(",");
              for (int i = 0; i < parts.length; i++)
              {
                parts[i] = parts[i].trim().replaceAll("", "").toLowerCase();
                curEntry.addWord(parts[i]);
                wordCount++;
                ArrayList check;

                check = (ArrayList)search.get(parts[i]);

                if (check == null)
                {
                  check = new ArrayList();
                }

                check.add(curEntry);

                search.put(parts[i], check);

              }
            }
          }
        in.close();
    }
    catch (IOException e) {}

  }

    /** Given the starting position on a JTree and a search word
     * this method will construct DefatultMutableTreeNodes containing JWordNodes
     * based on the dictionary's word relationships matched by the searchWord
     */
    public void  returnSearch(DefaultMutableTreeNode root, String searchWord)
    {
      DefaultMutableTreeNode posNode = new DefaultMutableTreeNode("Roots Found");
      DefaultMutableTreeNode curNode;

      ArrayList matches = (ArrayList)search.get(searchWord.toLowerCase());
      if (matches == null)
      {
        posNode = new DefaultMutableTreeNode("No Roots Found");
        root.add(posNode);
        return;
      }
      for (int i = 0; i < matches.size(); i++)
      {
        rootItem match = (rootItem)matches.get(i);
        //create main node, then add up to 10 words per underlying result
        curNode = new DefaultMutableTreeNode(new rootNode(match, searchWord));
        ArrayList wordsFound = match.words;
        String temp = "";
        for (int j = 0; j < wordsFound.size(); j++)
        {
          temp = temp + (String)wordsFound.get(j);

          if (j > 0 && j %10 == 0)
          {
            temp.replaceAll(searchWord, "<b><i>" + searchWord + "</i></b>");
            DefaultMutableTreeNode tail = new DefaultMutableTreeNode(temp);
            curNode.add(tail);
            temp = "";
            continue;
          }

          if (j != wordsFound.size() - 1)
            temp = temp + ", ";
        }

        if (!temp.equals(""))
        {
          temp.replaceAll(searchWord, "<b><i>" + searchWord + "</i></b>");
          DefaultMutableTreeNode tail = new DefaultMutableTreeNode(temp);
          curNode.add(tail);
        }
        posNode.add(curNode);
      }
      root.add(posNode);
    }

    /** Note: This Method is NOT SUPPORTED by the RootBookDriver!
     * Calling it will perform no action
     */
    public void returnPOS(DefaultMutableTreeNode posRoot, int POS, String searchWord)
    {
        System.out.println("NOT SUPPORTED BY THIS DRIVER");
    }

    public RootBookDriver()
    {

    }

    public static void main(String[] args)
    {
      RootBookDriver test = new RootBookDriver();
      test.initDictionary();
    }
}

class rootNode extends JWordNode
{

    private rootItem curMatch;
    private String curString;

    /** Builds a node based on the returned search results (curString) and
     * the word searched for (curWord)
     */
    public rootNode(rootItem match, String curWord)
    {
        //base(curWord, curString);
        this.curMatch = match;
        this.curString = curWord;
    }

    /** Returns curString with curWord bolded and italicized
     */
    public String toString()
    {

       String retStr = curMatch.root;
       retStr = retStr.substring(0, 1).toUpperCase() + retStr.substring(1);
       return ("<html>" + retStr.replaceAll(curString, "<b><i>" + curString + "</b></i>") + "</html>");
      // return retStrs[0] + " (" + sense.getWords().length + ")";
       //return sense.getDescription() + "- - -" + sense.getGloss();
       //return sense.getDescription();
    }

    /** Returns example sentences found in the RogetLibrary (none)
     */
    public String getDescription()
    {
      String retStr = "<Html><b><u>" + curMatch.root + "</u></b><BR>" + curMatch.description + "</HTML>";
      return retStr;
        /*String retStrs[] = sense.getGloss().split(";");
        retStrs[0] = retStrs[0].substring(0, 1).toUpperCase() + retStrs[0].substring(1);
        String retStr = retStrs[0].replaceAll(curWord, "<b><i>" + curWord + "</b></i>");
        retStr = "<html>" + retStr + "<UL>";

        for (int i = 1; i < retStrs.length; i++)
        {
            retStrs[i] = retStrs[i].substring(0, 1).toUpperCase() + retStrs[i].substring(1);
            retStr = retStr + "<LI>" + retStrs[i].replaceAll(curWord, "<b><i>" + curWord + "</b></i>") + "</LI>";
        }



        return retStr + "</UL></html>";*/

    }

    /** Returns example sentences found in the RogetLibrary (none)
     */
    public String getCurrentDescription()
    {
        /*String retStrs[] = sense.getGloss().split(";");
        retStrs[0] = retStrs[0].substring(0, 1).toUpperCase() + retStrs[0].substring(1);
        String retStr = retStrs[0].replaceAll(curWord, "<b><i>" + curWord + "</b></i>");
        retStr = "<html>" + retStr + "<UL>";

        for (int i = 1; i < retStrs.length; i++)
        {
            if (retStrs[i].indexOf(curWord) > -1)
            {
                retStrs[i] = retStrs[i].substring(0, 1).toUpperCase() + retStrs[i].substring(1);
                retStr = retStr + "<LI>" + retStrs[i].replaceAll(curWord, "<b><i>" + curWord + "</b></i>") + "</LI>";
            }
        }



        return retStr + "</UL></html>";*/
      String retStr = "<Html><b><u>" + curMatch.root + "</u></b><BR>" + curMatch.description + "</HTML>";
      return retStr;
    }


}


class rootItem
{
  public String root, description;
  public ArrayList words;
  public rootItem(String root, String Description)
  {
    this.root = root;
    this.description = Description.trim();
    words = new ArrayList();
  }

  public void addWord(String word)
  {
    words.add(word);

  }

  public String toString()
  {
     String retStr = root + "\n" + description + "\nWords:";
     for (int i = 0; i < words.size(); i++)
     {
       retStr = retStr + (String)words.get(i);
       if (i != words.size() - 1)
         retStr = retStr + ", ";
     }

     return retStr;
  }



}
