package edu.gwu.jworddrivers;

import edu.brandeis.cs.steele.wn.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.event.*;
import java.awt.event.MouseEvent;
import java.awt.Component;

/*
 * WordNetDriver.java
 *
 * Created on September 30, 2004, 10:19 PM
 */



/**
 *
 * @author  Kunal Johar
 */
public class WordNetDriver extends JWordDriver
{


    static DictionaryDatabase dictionary;

    public String getTitle()
    {
      return "Wordnet Synonym Relationships";
    }

    /** Load the dictionary file into memory */
    public  void initDictionary()
    {
        java.util.Properties p = new Properties();
        try
        {
          System.out.println(System.getProperty("user.dir"));
            java.io.FileInputStream f = new java.io.FileInputStream("wn.props");
            p.load(f);
        }
        catch (Exception e)
        {
            System.out.println("Dictionary Loader File not found");
            System.exit(0);
        }
        String dicFile = p.getProperty("dictionary.path");
        dicFile = dicFile.replace('\\', '/');

        //new FileBackedDictionary(searchDirectory
        dictionary = new FileBackedDictionary(dicFile);
    }

    /** This method is identical to that of returnSearch except that it generates
     * the tree starting at the Part of Speech level rather than the dictionary level
     */
    public  void returnPOS(DefaultMutableTreeNode posRoot, int POSv, String searchWord)
    {
        addMatches(POS.CATS[POSv], searchWord, posRoot);
    }

    /** Given a DefaultMutableTreeNode to start at and a word to search for
     * this will build the tree based on the WordNet's word relationship
     * using the
     */
    public  void  returnSearch(DefaultMutableTreeNode root, String searchWord)
    {
      //  DefaultMutableTreeNode root = new DefaultMutableTreeNode("Root");
        //DefaultTreeModel tm = new DefaultTreeModel(root);
        //DefaultMutableTreeNode curSenseNode;
        //DefaultMutableTreeNode posNode;



        //Query all parts of speech
        for (int i = 0; i < POS.CATS.length; i++)
           addMatches(POS.CATS[i], searchWord, root);

    }

    private  void addMatchesPOS(POS type, String searchWord, DefaultMutableTreeNode root)
    {
        //create storage on the tree
        DefaultMutableTreeNode posNode = root;
        DefaultMutableTreeNode curSenseNode;

        IndexWord word; //get the word

        //command to search the DB (Part of Speech, Word)
        word = dictionary.lookupIndexWord(type, searchWord);

        //if not found
        if (word == null)
           return;

        //System.out.println(word.getLemma());

        //Else display
        Synset[] senses;
        try
        {
           senses = word.getSenses();
        }
        catch(Exception e)
        {
              return;
        }


        //for each word,
        //create a WordNetNode which will break the gloss up
        //one for the tree, one for the GlossBar
        for (int i = 0; i < senses.length; i++)
        {
            Synset sense = senses[i];
            if (sense == null)
                continue;

            //System.out.println("Num Senses: " + senses.length);
            Word[] printWords = sense.getWords();
            //System.out.println(sense.getGloss());
            curSenseNode = new DefaultMutableTreeNode(new WordNetNode(sense, searchWord));

            for (int j = 0; j < printWords.length; j++)
            {
                String curLemma = printWords[j].getLemma();
                curLemma = curLemma.substring(0, 1).toUpperCase() + curLemma.substring(1);
                if (printWords[j].getLemma().equals(searchWord))
                {
                    curSenseNode.add(new DefaultMutableTreeNode("<html><b><i>" + curLemma + "</b></i></html>"));
                }
                else
                    curSenseNode.add(new DefaultMutableTreeNode(curLemma));
            }
            posNode.add(curSenseNode);

        }
    }


    private  void addMatches(POS type, String searchWord, DefaultMutableTreeNode root)
    {
        //create storage on the tree
        DefaultMutableTreeNode posNode;
        DefaultMutableTreeNode curSenseNode;

        IndexWord word; //get the word

        //command to search the DB (Part of Speech, Word)
        word = dictionary.lookupIndexWord(type, searchWord);

        //if not found
        if (word == null)
           return;

        //System.out.println(word.getLemma());

        //Else display
        Synset[] senses;
        try
        {
	   senses = word.getSenses();
        }
        catch(Exception e)
        {
              return;
        }


        posNode = new DefaultMutableTreeNode(type.getLabel().substring(0,1).toUpperCase() + type.getLabel().substring(1) + " Matches");

        //for each word,
        //create a WordNetNode which will break the gloss up
        //one for the tree, one for the GlossBar
        for (int i = 0; i < senses.length; i++)
        {
            Synset sense = senses[i];
            if (sense == null)
                continue;

            //System.out.println("Num Senses: " + senses.length);
            Word[] printWords = sense.getWords();
            //System.out.println(sense.getGloss());
            curSenseNode = new DefaultMutableTreeNode(new WordNetNode(sense, searchWord));

            for (int j = 0; j < printWords.length; j++)
            {
                String curLemma = printWords[j].getLemma();
                curLemma = curLemma.substring(0, 1).toUpperCase() + curLemma.substring(1);
                if (printWords[j].getLemma().equals(searchWord))
                {
                    curSenseNode.add(new DefaultMutableTreeNode("<html><b><i>" + curLemma + "</b></i></html>"));
                }
                else
                    curSenseNode.add(new DefaultMutableTreeNode(curLemma));
            }
            posNode.add(curSenseNode);

        }
        if (posNode.children().hasMoreElements())
          root.add(posNode);
    }

    /** Returns the WordNet's 'lemma' of a word.  For example
     * Weekends would return Weekend and Clearly would return Clear
     */
    public  String getLemma(String word, int pos)
    {
      String retWord = null;
        for (pos = 0; pos < 4; pos++)
        {
           POS type= POS.ADJ;
           switch(pos)
           {
               case JWordDriver.POS_NOUN:
                 type = POS.NOUN;
                 break;
               case 1:
                 type = POS.VERB;
                 break;
               case 2:
                 type = POS.ADJ;
                 break;
               case 3:
                 type = POS.ADV;
                 break;
           }


        retWord = dictionary.lookupBaseForm(type,  word);
        if (retWord != null)
         break;
     }


        //command to search the DB (Part of Speech, Word)
        return retWord;

    }

    /** Returns WordNet's example sentences that are in the SynSet of the lemma
     * of the word, and where the word is found in the example sentence
     */
    public static Object[] getExampleSentences(String word, int pos)
    {
        ArrayList retStr = new ArrayList();
        POS type= POS.ADJ;
        switch(pos)
        {
            case JWordDriver.POS_NOUN:
              type = POS.NOUN;
              break;
            case 1:
              type = POS.VERB;
              break;
            case 2:
              type = POS.ADJ;
              break;
            case 3:
              type = POS.ADV;
              break;
        }

        IndexWord w = dictionary.lookupIndexWord(type, word);
        Synset[] senses = null;

        try
        {
          senses = w.getSenses();
        }
        catch(Exception e)
        {
        }

        for (int i = 0; i < senses.length; i++)
        {
          Synset s = senses[i];
          String[] st = s.getGloss().split(";");

          for (int j = 0; j < st.length; j++)
          {
            retStr.add(st[j]);
          }
        }
        if (retStr.size() > 0)
        return retStr.toArray();
      return null;
    }


    public static void main(String args[])
    {
      WordNetDriver wn = new WordNetDriver();
      wn.initDictionary();
      System.out.println(wn.getLemma("holds", JWordDriver.POS_VERB));
      Object[] s = wn.getExampleSentences("Hold", JWordDriver.POS_VERB);
      for (int i = 0; i < s.length; i++)
      {
        System.out.println(s[i]);
      }

    }

}
