package edu.gwu.jworddrivers;


import edu.brandeis.cs.steele.wn.*;




/** WordNetNode Node is a class used for displaying <b>Syn</b>onym <b>Set</b>s in a JTree
 * and additional information text component.  This component is an additional
 * abstraction from the Wordnet library, it adds no additional functionality.
 */
public class WordNetNode extends JWordNode
{
    private Synset sense;
    private String curWord;

    /** Takes in a Synset to create object.
     * @param sense Input Synset to break into its String components.
     */
    public WordNetNode(Synset sense, String curWord)
    {
        this.sense = sense;
        this.curWord = curWord;
    }

    public WordNetNode()
    {
    }


    /** Returns a string containing the first Gloss found for the sense.  In addition it
     * returns the number of words contained by the synset.
     */
    public String toString()
    {
		/* Code modified by K.  Johar for use with JWord.
		   Original code appears in Example Usage File 				   http://jwn.sourceforge.net/

	   Modification displays search results using formatted HTML. Original code displays as text.
		*/
       String retStrs[] = sense.getGloss().split(";");
       retStrs[0] = retStrs[0].substring(0, 1).toUpperCase() + retStrs[0].substring(1);
       return ("<html>" + retStrs[0].replaceAll(curWord, "<b><i>" + curWord + "</b></i>") + "</html>");
      // return retStrs[0] + " (" + sense.getWords().length + ")";
       //return sense.getDescription() + "- - -" + sense.getGloss();
       //return sense.getDescription();
    }

    /** Returns an HTML formatted list of each gloss found for the
     * synset.
     */
    public String getDescription()
    {
        String retStrs[] = sense.getGloss().split(";");
        retStrs[0] = retStrs[0].substring(0, 1).toUpperCase() + retStrs[0].substring(1);
        String retStr = retStrs[0].replaceAll(curWord, "<b><i>" + curWord + "</b></i>");
        retStr = "<html>" + retStr + "<UL>";

        for (int i = 1; i < retStrs.length; i++)
        {
            retStrs[i] = retStrs[i].substring(0, 1).toUpperCase() + retStrs[i].substring(1);
            retStr = retStr + "<LI>" + retStrs[i].replaceAll(curWord, "<b><i>" + curWord + "</b></i>") + "</LI>";
        }



        return retStr + "</UL></html>";
    }

    /** Returns an HTML formatted list of each gloss found for the
     * current selected word
     */
    public String getCurrentDescription()
    {
        String retStrs[] = sense.getGloss().split(";");
        retStrs[0] = retStrs[0].substring(0, 1).toUpperCase() + retStrs[0].substring(1);
        String retStr = retStrs[0].replaceAll(curWord, "<b><i>" + curWord + "</b></i>");
        retStr = "<html>" + retStr + "<UL>";

        for (int i = 1; i < retStrs.length; i++)
        {
            if (retStrs[i].indexOf(curWord) > -1)
            {
                retStrs[i] = retStrs[i].substring(0, 1).toUpperCase() + retStrs[i].substring(1);
                retStr = retStr + "<LI>" + retStrs[i].replaceAll(curWord, "<b><i>" + curWord + "</b></i>") + "</LI>";
            }
        }



        return retStr + "</UL></html>";
    }


}
